/***************************************************************************
                          level.c  -  description
                             -------------------
    begin                : Sat Apr 5 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <string.h>
#include <ctype.h>
#include "explore.h"
#include "level.h"
#include "render.h"
#include "objects.h"

#define EXPLORER_DUG_LAZY_REPAIR_SCREEN		0	/* try as 1 to see how slow graphics can be! */

char mainmap[] = {
"++++++++++++++++++++"
"+    1     1       +"
"+   >         2  / +"
"+      vvvv)  k    +"
"+     k   2------- +"
"+  *--             +"
"+ ^^  )       1  v-+"
"+   -------      v1+"
"+vvv    >    k   v-+"
"+           ---  v-+"
"+        >  1     }+"
"+     -v----//   wx+"
"+A     ^         yz+"
"++++++++++++++++++++"
};

/*
** Level Admin
*/
static int get_tile(int map)
{
	switch(map)
		{
		case	'+':	return 1;
		case	'-':	return 2;
		case	'*':	return 3;
		case	'/':	return 4;
		case	'^':	return 8;
		case	'w':	return 65;
		case	'x':	return 66;
		case	'y':	return 67;
		case	'z':	return 68;
		}
return 0;
}

BOOL exLoadLevel(int level)
{
int tx,ty;
int x,y;
int idx;
EX_TILE *tile_ptr;
EX_OBJECT *pO;

	/* Create static object map */
	idx = 0;
	tile_ptr = TheGame.curr_level.pTileMap;
	for(ty=0;ty<TheGame.iNumTileHeight;ty++)
		for(tx=0;tx<TheGame.iNumTileWidth;tx++)
			{
			tile_ptr->iRegion = get_tile(mainmap[idx]);
			tile_ptr++;
			idx++;
			}

	/* Create dynamic objects */
	TheGame.curr_level.pObjList = NULL;

	idx = 0;
	for(ty=0,y=0;ty<TheGame.iNumTileHeight;ty++,y+=TheGame.iTileHeight)
		for(tx=0,x=0;tx<TheGame.iNumTileWidth;tx++,x+=TheGame.iTileWidth)
			{
			if (mainmap[idx] == ')')	pO = exCreateObject(x, y, 2, 5);	
			if (mainmap[idx] == '>')	pO = exCreateObject(x, y, 3, 0);	
			if (mainmap[idx] == '}')	pO = exCreateObject(x, y, 5, 5);	

			idx++;
			}

	TheGame.curr_level.pBackdrop = SDL_LoadBMP("gfx/backdrop.bmp");

return TRUE;
}

/*
** Level updates
*/
void exStartLevel(void)
{
}

void exResetLevel(void)
{
EX_OBJECT *pObj;

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		pObj->x = pObj->init_x;
		pObj->y = pObj->init_y;
		pObj = pObj->pNext;
		}
}

void exDestroyLevel(void)
{
EX_OBJECT *pObj;

	/* Level */
	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		pObj = exDestroyObject(pObj);

	/* Gfx */
	SDL_FreeSurface(TheGame.curr_level.pBackdrop);
}

void exUpdateLevel(int frames)
{
EX_OBJECT *pObj;

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		exUpdateObject(pObj);
		pObj = pObj->pNext;
		}
}

/*
** Draw
*/
void exDrawStatic(void)
{
int tx, ty;

	for(ty=0;ty<TheGame.iNumTileHeight;ty++)
		for(tx=0;tx<TheGame.iNumTileWidth;tx++)
			exDrawLevelTile(tx, ty);
}

void exRepairScreen(void)
{
/* My 'top' goes from 19% to 44% of %CPU with the lazy method... */
#if EXPLORER_DUG_LAZY_REPAIR_SCREEN
	exDrawStatic();
#else
EX_OBJECT *pObj;
int tx, ty;

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		/*
		 Repair everything, because we might need the backdrop
		 for animating (static) objects that have alpha
		*/
		tx = pObj->x / TheGame.iTileWidth;
		ty = pObj->y / TheGame.iTileHeight;

		exDrawLevelTile(tx, ty);
		if (pObj->x % TheGame.iTileWidth)
			exDrawLevelTile(tx+1, ty);
		if (pObj->y % TheGame.iTileHeight)
			exDrawLevelTile(tx, ty+1);
		if ((pObj->x % TheGame.iTileWidth) && (pObj->y % TheGame.iTileHeight))
			exDrawLevelTile(tx+1, ty+1);

		pObj = pObj->pNext;
		}
#endif
}


void exDrawDynamic(void)
{
EX_OBJECT *pObj;
Uint8 old_alpha;
static int iAmountOfAlpha=0;

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		exDrawObject(pObj);
		pObj = pObj->pNext;
		}


	/* Draw a player, and fade him in! */
	old_alpha = TheGame.pPlayerGfx->format->alpha;
	SDL_SetAlpha(TheGame.pPlayerGfx, SDL_SRCALPHA, iAmountOfAlpha);

	exDrawTile(TheGame.pPlayerGfx, 0, 32, 384);

	SDL_SetAlpha(TheGame.pPlayerGfx, SDL_SRCALPHA, old_alpha);
	iAmountOfAlpha += 2;
	if (iAmountOfAlpha > 255)
		iAmountOfAlpha = 255;
}

EX_TILE *exGetTileAt(int tx, int ty)
{
	if (tx < 0 || ty < 0)
		return NULL;
	if (tx >= TheGame.iNumTileWidth || ty >= TheGame.iNumTileHeight)
		return NULL;

	return &TheGame.curr_level.pTileMap[tx + ty*TheGame.iNumTileWidth];
}
